/*

Additions:

	atom/movable
		proc
			PixelMove(move_x, move_y)
			PixelMove(vector2/move)

			PixelSlide(move_x, move_y)
			PixelSlide(vector2/move)

			PixelMoveTo(center_x, center_y)
			PixelMoveTo(vector2/center)
			PixelMoveTo(atom/center)

			PixelSlideTo(center_x, center_y)
			PixelSlideTo(vector2/center)
			PixelSlideTo(atom/center)

			SetLowerTo(lower_x, lower_y, z)
			SetLowerTo(vector2/lower, z)
			SetLowerTo(atom/lower)

			SetCenterTo(center_x, center_y, z)
			SetCenterTo(vector2/center, z)
			SetCenterTo(atom/center)

Deprecated:

	atom/movable
		proc
			SetLowerPosition(center_x, center_y, z)
			SetLowerPosition(vector2/center, z)
			SetLowerPosition(atom/center)

			SetCenterPosition(center_x, center_y, z)
			SetCenterPosition(vector2/center, z)
			SetCenterPosition(atom/center)

*/
atom/movable
	proc
		PixelMove(move_x, move_y)
			/* Move along a line according to the given pixel offsets.

				Returns Move() and changes step_size.

				Overloads:
					PixelMove(vector2/move)
			*/
			if(istype(move_x, /vector2))
				var vector2/v = move_x
				move_x = v.x
				move_y = v.y

			if(!(move_x || move_y))
				return 0

			// Enable sliding to anywhere in the world.
			step_size = 1#INF

			// Move without changing dir and return the result.
			. = Move(loc, dir, step_x + move_x, step_y + move_y)

			// Set step_size to the amount actually moved.
			// This tells clients to smoothly interpolate this when using client.fps.
			step_size = 1 + .

		PixelSlide(move_x, move_y)
			/* Attempt to move along the line.
				If that fails, try moving in one axis at a time.

				Returns equivalent PixelMove().

				Overloads:
					PixelSlide(vector2/move)
			*/
			if(istype(move_x, /vector2))
				var vector2/v = move_x
				move_x = v.x
				move_y = v.y

			if(!(move_x || move_y))
				return 0

			// Move straight
			. = PixelMove(move_x, move_y)
			if(!.)
				// Or move horizontally
				. = PixelMove(move_x, 0)

				// Then move vertically
				if(.)
					. = max(., PixelMove(0, move_y))

				// Or move vertically, then horizontally
				else
					. = max(PixelMove(0, move_y), PixelMove(move_x, 0))

			// Set step size to the amount actually moved. (See PixelMove)
			step_size = 1 + .

		PixelMoveTo(center_x, center_y)
			/* Move in a straight line to a center position on the same z-level.

				Returns equivalent PixelMove().

				Overloads:
					PixelMoveTo(vector2/center)
					PixelMoveTo(atom/center)
			*/

			if(istype(center_x, /vector2))
				var vector2/center = center_x
				center_x = center.x
				center_y = center.y

			else if(istype(center_x, /atom))
				var atom/center = center_x
				center_x = center.CenterX()
				center_y = center.CenterY()

			return PixelMove(center_x - CenterX(), center_y - CenterY())

		PixelSlideTo(center_x, center_y)
			/* Move in a straight line to a center position on the same z-level.

				Returns equivalent PixelSlide().

				Overloads:
					PixelSlideTo(vector2/center)
					PixelSlideTo(atom/center)
			*/

			if(istype(center_x, /vector2))
				var vector2/center = center_x
				center_x = center.x
				center_y = center.y

			else if(istype(center_x, /atom))
				var atom/center = center_x
				center_x = center.CenterX()
				center_y = center.CenterY()

			return PixelSlide(center_x - CenterX(), center_y - CenterY())

		SetLowerTo(lower_x, lower_y, z)
			/* Set the lower corner of this movable to a world pixel position.
				Does not call Move() or any side-effects such as Entered().

				Overloads:
					SetLowerTo(vector2/lower, z)
					SetLowerTo(atom/lower)

				z defaults to src.z.
			*/
			if(istype(lower_x, /vector2))
				var vector2/v = lower_x
				z = lower_y
				lower_x = v.x
				lower_y = v.y

			else if(istype(lower_x, /atom))
				var atom/a = lower_x
				lower_x = a.LowerX()
				lower_y = a.LowerY()
				z = a.z

			if(!z) z = src.z

			var
				tile_x = 1 + round((lower_x - 1) / TileWidth)
				tile_y = 1 + round((lower_y - 1) / TileHeight)
				step_x = (lower_x - 1) - (tile_x - 1) * TileWidth
				step_y = (lower_y - 1) - (tile_y - 1) * TileHeight

			// Normalize to within the map edges
			if(tile_x < 1)
				step_x += TileWidth * (tile_x - 1)
				tile_x = 1
			if(tile_x > world.maxx)
				step_x += TileWidth * (tile_x - world.maxx)
				tile_x = world.maxx
			if(tile_y < 1)
				step_y += TileHeight * (tile_y - 1)
				tile_y = 1
			if(tile_y > world.maxy)
				step_y += TileHeight * (tile_y - world.maxy)
				tile_y = world.maxy

			loc = locate(tile_x, tile_y, z)
			src.step_x = step_x - bound_x
			src.step_y = step_y - bound_y

		SetLowerPosition()
			// Deprecated: Renamed to SetLowerTo.
			SetCenterTo(arglist(args))

		SetCenterTo(center_x, center_y, z)
			/* Sets the center of this movable to a world pixel position.
				Does not call Move() or any side-effects such as Entered().

				Overloads:
					SetCenterTo(vector2/center, z)
					SetCenterTo(atom/center)

				z defaults to src.z.

			*/
			if(istype(center_x, /vector2))
				var vector2/center = center_x
				z = center_y
				center_x = center.x
				center_y = center.y

			else if(istype(center_x, /atom))
				var atom/center = center_x
				center_x = center.CenterX()
				center_y = center.CenterY()
				z = center.z

			SetLowerTo(
				center_x - bound_width / 2,
				center_y - bound_height / 2,
				z)

		SetCenterPosition()
			// Deprecated: Renamed to SetCenterTo.
			SetCenterTo(arglist(args))
